import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.KeyStroke;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;

/***
 * @description	This class contains behaviour all slides have in common, like the
 * action bindings for the navigation logic, and timing via ticks.
 */
public abstract class BasicSlide extends JPanel {
	
	private static final long serialVersionUID = 1L;
	SlideShow show;
	protected int tick, rate;

	public boolean changed = true;
	
	private boolean overlay = false;
	private Font debugfont;
	long thischeck, lastcheck, frames, nextframes;
	
	final protected int defaultrate = 16;
	
	public abstract void render(Graphics g);
	public abstract void cycle();

	public BasicSlide(SlideShow s){
		
		show = s;
		setBackground(Color.BLACK);
		
		rate = defaultrate;
		
		lastcheck = System.currentTimeMillis();
		
		debugfont = new Font("Serif", Font.PLAIN, 12);
		
		Action next = new NavigationAction(NavigationAction.NEXT_SLIDE);
		this.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke("SPACE"), "nextSlide");
		//this.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke("ENTER"), "nextSlide");
		this.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke("F2"), "nextSlide");
		this.getActionMap().put("nextSlide", next);
		Action prev = new NavigationAction(NavigationAction.PREVIOUS_SLIDE);
		this.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke("F1"), "prevSlide");
		this.getActionMap().put("prevSlide", prev);
		Action prevC = new NavigationAction(NavigationAction.PREVIOUS_CHAPTER);
		this.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke("F3"), "prevChapter");
		this.getActionMap().put("pervChapter", prevC);
		Action nextC = new NavigationAction(NavigationAction.NEXT_CHAPTER);
		this.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke("F4"), "nextChapter");
		this.getActionMap().put("nextChapter", nextC);
		Action reload = new NavigationAction(NavigationAction.RELOAD_SLIDE);
		this.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke("F5"), "reloadSlide");
		this.getActionMap().put("reloadSlide", reload);
		Action first = new NavigationAction(NavigationAction.FIRST_SLIDE);
		this.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke("F6"), "firstSilde");
		this.getActionMap().put("firstSilde", first);
		Action exit = new NavigationAction(NavigationAction.EXIT_SHOW);
		this.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke("ESCAPE"), "exitShow");
		this.getActionMap().put("exitShow", exit);
	}
	
	public void paint(Graphics g){
		super.paint(g);
		render(g);
		if (overlay) paintOverlay(g);
		Toolkit.getDefaultToolkit().sync();
		g.dispose();
		changed = false;
	}
	
	private void paintOverlay(Graphics g) {
		g.setFont(debugfont);
		g.setColor(Color.WHITE);
		thischeck = System.currentTimeMillis();
		g.drawString(frames + " FPS", 10, 10);
		nextframes++;
		if(thischeck - lastcheck > 1000){
			frames = nextframes;
			nextframes = 0;
			lastcheck = thischeck;
		}
		
	}
	public void tick()
	{
		if(++tick >= rate)
		{
			tick = 0;
			cycle();
		}
	}
	
	class NavigationAction extends AbstractAction {
		static final int PREVIOUS_SLIDE = 0;
		static final int NEXT_SLIDE = 1;
		static final int PREVIOUS_CHAPTER = 2;
		static final int NEXT_CHAPTER = 3;
		static final int RELOAD_SLIDE = 4;
		static final int FIRST_SLIDE = 5;

		static final int EXIT_SHOW = 6;
		private static final long serialVersionUID = 1L;
		
		int type;
		
		public NavigationAction(int t){
			type = t;
		}
		
		public void actionPerformed(ActionEvent e) {
			show.navigate(type);
		}
	}
}
