import java.util.ArrayList;
import javax.swing.JFrame;


/*
\\for sceenshot necessary imports
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
*/


/***
 * This class manages the slideshow. It contains the main loop, in which the current slide
 * is rendered approximately every DELAY ms, and also the navigation logic which is called
 * by the individual slides. The presentation is built in the constructor.
 *
 */
public class SlideShow implements Runnable {
	
	ArrayList<BasicSlide> slides = new ArrayList<BasicSlide>();
	int current, next = 0;
	BasicSlide slide;
	JFrame frame;
	private final int DELAY = 50;
	int xsize, ysize;
	boolean reload;
	Integer scrshotindex = 0;
	
	public SlideShow (JFrame f, int width, int height){
		
		xsize = width;
		ysize = height;
		
		//color tables
		int[] coltable =  {255<<16,255<<8,255};
		int[] rndcol  =  {0,255,255<<8};
		int[] hppgas =  {0, Integer.MAX_VALUE};
		int[] margolous =  {255<<16,255<<8,128<<16,128<<8};
		int[] parityflip =  {0, Integer.MAX_VALUE, (16<<16)+(16<<8)+(16),Integer.MAX_VALUE};
		
/*01*/	slides.add(new ImageSlide(this,"keymap.png"));
/*02*/	slides.add(new TitleSlide(this,"<html>Zellul&#228re Automaten</html>"));
/*03*/	slides.add(new TextSlide(this,"<html> &#220berblick:</html>", "<html> <p><p>\u25aa Definition <p><p>\u25aa Geschichtlich wichtige Personen <p><p>\u25aa Anwendungen <p><p>\u25aa Charakteristika <p><p>\u25aa Eigenschaften <p><p>\u25aa Beispiele </html>"));
/*04*/	slides.add(new GameOfLife(this, new CellSpace(32,24,6,coltable, true)));
/*05*/	slides.add(new TextSlide(this, "Definition (lt. von Neumann)", "<html><p><p>\u25aa Netzwerk aus interagierenden Zellen <p><p>\u25aa Zelle kann einen von n m&#246glichen Zust&#228nden annehmen <p><p>\u25aa konkrete Zeitdynamik <p><p>Der n&#228chste Zellzustand ist jeweils abh&#228ngig von den<br>aktuellen Zust&#228nden der Nachbarzellen</html>"));
/*06*/	slides.add(new TextSlide(this,"Formale Beschreibung","<html><p><p>\u25aa ein Raum R<p><p> \u25aa eine endliche Nachbarschaft&nbsp&nbsp N <p><p>\u25aa eine Zustandsmenge&nbsp Q<p><p> \u25aa eine &#220berf&#252hrungsfunktion&nbsp&nbsp&nbsp&#948: Q<sup><font size=\"6\">N</font></sup> 	&#8594 Q </html>"));
/*07*/	slides.add(new ImageSlide(this, "game_of_life.png"));
/*08*/	slides.add(new GameOfLife(this, new CellSpace(32,24,6,coltable, true)));
/*09*/	slides.add(new TextSlide(this,"Geschichtlich wichtige Personen","<html> <p><p> \u25aa Stanislaw Marcin Ulam<p><p> \u25aa John Von Neumann<p><p> \u25aa Aristid Lindenmayer<p><p> \u25aa Konrad Zuse<p><p> \u25aa John Horton Conway<p><p> \u25aa Stephen Wolfram</html"));
/*10*/	slides.add(new TextSlide(this,"Anwendungen","<html> <p><p> Untersuchung von Interaktionen oder<br> Modellierung r&#228umlicher Dynamiken <p><p>\u25aa Emissionsausbreitungsmodelle <p><p>\u25aa Ausbreitung von Waldbr&#228nden <p><p> \u25aa Modelle st&#228dtischem Bev&#246kerungswachstums <p><p> \u25aa Interaktion von Pflanzen mit ihrer Umwelt  <p><p> \u25aa uvm.</html>"));
/*11*/	slides.add(new TextSlide(this,"Charakteristika", "<html><p><p> \u25aa Geometrie des Zellraumes <p><p> \u25aa Regelwerk <p><p> \u25aa Zust&#228nde <p><p> \u25aa Nachbarschaften </html> "));
/*12*/	slides.add(new TextSlide(this,"Geometrie","<html><p><p> \u25aa verschieden viele Dimensionen <br> (meist 1-, 2- oder 3-dimensional) <p><p> \u25aa Anordnung: rechteckig, hexagonal, dreieckig ... <p><p> \u25aa endlicher oder unendlicher Raum </html> "));
		//slides.add(new TextSlide(this, "Randzellen" , "<html> <p><p>nicht alle Nachbarzellen vorhanden <p><p> L&#246sungen: <p><p> \u25aa vordefinierte,fiktive Nachbarzellen <p><p> \u25aa R&#228nder \"verkleben\" (periodische L&#246sung)<p><p> \u25aa Spiegelung (symmetrische L&#246sung) </html> "));
/*13*/	slides.add(new ImageSlide(this,"borders.png"));
/*14*/	slides.add(new TextSlide(this,"Regelwerk","<html> <p><p> \u25aa steuert Ver&#228nderungen<p><p> \u25aa deterministische Regeln </html> "));
/*15*/	slides.add(new TextSlide(this,"<html>Zust&#228nde</html>","<html><p><p> \u25aa Anzahl h&#228ngt vom Problem ab <p><p> \u25aa polygene Automaten </html>"));
		//slides.add(new TextSlide(this,"Nachbarschaften","<html><p><p> \u25aa Von Neumann-Nachbarschaft <p><p> \u25aa Moore-Nachbarschaft <p><p> \u25aa Input-, Outputnachbarschaften <p><p> \u25aa Blocknachbarschaft</html> "));
/*16*/	slides.add(new ImageSlide(this,"neighbourhoods.png"));
/*17*/	slides.add(new TextSlide(this,"Blocknachbarschaft", "<html> <p><p> \u25aa Zellraum wird in Bl&ouml;cke eingeteilt <p><p> \u25aa Einteilung wechselt bei jedem Schritt<p><p> \u25aa Margolus-Nachbarschaft (Blockgr&#246&#223e 2x2) </html> "));
/*18*/	slides.add(new ImageSlide(this,"blockrule.png"));
/*19*/	slides.add(new MargolousDemo(this, new CellSpace(32,24, 6, margolous, true)));
/*20*/	slides.add(new TextSlide(this,"Ausgangskonfiguration:", "<html> <p><p> \u25aa k&#252nstlich konstruieren <p><p> \u25aa zuf&#228llig herstellen <p><p> \u25aa reale Daten z.B. Kartenmaterial </html>" ));
/*21*/	slides.add(new TextSlide(this,"Eigenschaften","<html> <p><p> \u25aa Lokalit&#228t<p><p>\u25aa Linearit&#228t<p><p>\u25aa Umkehrbarkeit</html>"));
/*22*/	slides.add(new SubTitleSlide(this,"Lokalit\u00e4t"));
/*23*/	slides.add(new ImageSlide(this, "neighbour_active.png"));
/*24*/	slides.add(new Growth(this, new CellSpace(32,24,6,rndcol, true)));
		//slides.add(new SlowGrowth(this, new CellSpace(512,384,0,hppgas, false)));
/*25*/	slides.add(new SubTitleSlide(this,"Linearit\u00e4t"));
/*26*/	slides.add(new ImageSlide(this, "parity.png"));
/*27*/	slides.add(new Parity(this, new CellSpace(512,384,0,hppgas, false)));
/*28*/	slides.add(new ParityScripted(this, new CellSpace(512,384,0,hppgas, false)));
/*29*/	slides.add(new SubTitleSlide(this,"Umkehrbarkeit"));
/*30*/	slides.add(new ImageSlide(this, "modified_parity.png"));
		//slides.add(new TextSlide(this,"<html> Anmerkung:</html>", "<html> <p><p>Im folgenden Beispiel werden zuerst die ersten<br>20 Schritte des Automaten gezeigt,<br> wobei immer nur die Gegenwart dargestellt wird.<p><p>Danach werden Vergangenheit und Gegenwart<br> vertauscht, und der Automat beginnt in der Zeit<br> r&uuml;ckw&auml;rts zu laufen.</html>"));
/*31*/	slides.add(new ParityFlip(this, new CellSpace(512,384,0,parityflip, false)));
/*32*/	slides.add(new TextSlide(this,"Beispiel 1 - Zufallszahlen","<html> <p><p>2006: Evolutionary Design of Pseudorandom<br>Sequence Generators based on<br>Cellular Automata and Its Applicability<br>in Current Cryptosystems<br>&nbsp&nbsp von Delgado, Vidal, Hernandez<p><p> \u25aa Eindimensionale 5-Nachbarn-Regel<p><p> \u25aa RULE 2572018122</html>"));
		//slides.add(new TextSlide(this,"RULE 2572018122","<html> <p><p> Regelbeschreibungen k&ouml;nnen als Zahlen ausgedr&uuml;ckt<br>werden. Dabei wird Vorrausgesetzt, dass die<br>vorgegebene Nachbarschaft bekannt ist, und die Anzah<br> der m&ouml;glichen Zellzust&auml;nde zwei ist.<br>Die m&ouml;glichen Eingangszust&auml;nde werden in einer bestimmten<br>konventionell vorgegebenen Reihenfolge aufgelistet," +
				//"und jedes Stelle der bin&auml;ren Zahl wird als Ausgangswert f&uuml; die Tabellarisch an gleicher Stelle stehende Ausgangskonfiguration gew&auml;hlt. </html>"));
/*33*/	slides.add(new ImageSlide(this,"rule_num1.png"));
/*34*/	slides.add(new ImageSlide(this,"rule_num2.png"));
/*35*/	slides.add(new RandomNumber(this, new CellSpace(32,24,6,rndcol, true)));
/*36*/	slides.add(new TextSlide(this,"Beispiel 2 - HPP-GAS","<html><p><p>Simulation des Verhaltens von Gaspartikeln<br>mit ungleichm&#228;&#223iger Raumverteilung.</html>"));
/*37*/	slides.add(new ImageSlide(this,"blockrule2.png"));
/*38*/	slides.add(new HPPGas(this, new CellSpace(512,384,0,hppgas, false)));
/*39*/	slides.add(new TextSlide(this,"Zu Beispiel 2","<html> <p><p> \u25aa Elastisches Medium  <p><p> \u25aa auf maktroskopischer Ebene nicht anisotropisch <p><p> \u25aa Schallgeschwindigkeit kleiner als Partikelgeschwindikeit <p><p> \u25aa verletzt allerdings NavierStokes Gleichung <p><p> Dieser Gleichung folgt das FHP-GAS Modell</html>"));
		frame = f;
	}
	
	@Override
	public void run() {
		long before, diff, sleep;
		
		before = System.currentTimeMillis();
		
		while (true)
		{
			if (reload){
				//slides.set(new DemoCellular(this,30,20, 10));
				reload = false;
			}
			slide = slides.get(next);
			current = next;
			frame.add(slide);
			frame.validate();
			slide.changed = true;
			while (current == next && !reload) {
				slide.tick();
				if(slide.changed)slide.repaint();
				
				diff = System.currentTimeMillis() - before;
				sleep = DELAY - diff;
				
				if (sleep < 0) sleep = 2;
				
				try {
					Thread.sleep(sleep);
				} catch (InterruptedException e){}
				
				before = System.currentTimeMillis();
			}
			frame.remove(slide);
		}
		
	}
	
	public void navigate(int type)
	{
		switch (type){
			case BasicSlide.NavigationAction.NEXT_SLIDE:
				if(current < slides.size() - 1)next = current + 1;
				break;
			case BasicSlide.NavigationAction.PREVIOUS_SLIDE:
				if(next > 0) next = current - 1;
				break;
			case BasicSlide.NavigationAction.PREVIOUS_CHAPTER:
				if(next > 0) next = current - 1;
				break;
			case BasicSlide.NavigationAction.NEXT_CHAPTER:
				if(current < slides.size() - 1)next = current + 1;
				
			/*
				// screenshot code for slide production
				BufferedImage screenshot = new BufferedImage(xsize,ysize,BufferedImage.TYPE_INT_RGB);
				Graphics2D g = screenshot.createGraphics();
				slide.paint(g);
				File output = new File( String.format("%09d", scrshotindex) + ".png");
				System.out.println("writing: " + String.format("%09d", scrshotindex) + ".png");
				scrshotindex++;
			try {
				ImageIO.write(screenshot, "png", output);
			} catch (IOException e) {
				System.out.println("Woopsie, lets pretend that didn't happen. screenshot failed");
				e.printStackTrace();
			}
			
			*/
				break;
			case BasicSlide.NavigationAction.RELOAD_SLIDE:
				reload = true;
				break;
			case BasicSlide.NavigationAction.FIRST_SLIDE:
				next = 0;
				break;
			case BasicSlide.NavigationAction.EXIT_SHOW:
				System.exit(0);
		}
	}
	

}
